package ch.epfl.collections;

import java.util.Iterator;

/**
 * Same generic interface as in the first exercise. Major changes are the implementation
 * of the generic Iterable<E> interface and switch to the usage of generics.
 * 
 * @see #iterator()
 *
 * @param <E>
 */
public interface List<E> extends Iterable<E> {
    /**
     * Check whether the list is empty.
     * @return <code>true</code> if and only if the list is empty.
     */
    boolean isEmpty();

    /**
     * Get the number of elements contained in the list.
     * @return the size of the list
     */
    int size();

    /**
     * Add an element at the end of the list.
     * @param newElem the element to add.
     */
    void add(E newElem);

    /**
     * Remove the element at a given position.
     * @param index the 0-based index of the element to remove.
     * @throws java.lang.IndexOutOfBoundsException if the index is invalid (i.e. negative or greater or equal to the size of the list)
     */
    void remove(int index);

    /**
     * Get the element at a given position.
     * @param index the 0-based index of the element to get.
     * @return the element at the given index.
     * @throws java.lang.IndexOutOfBoundsException if the index is invalid (i.e. negative or greater or equal to the size of the list)
     */
    E get(int index);

    /**
     * Replace the element at a given position by a new element.
     * @param index the 0-based index of the element to set.
     * @param newElem the new value of the element.
     * @throws java.lang.IndexOutOfBoundsException if the index is invalid (i.e. negative or greater or equal to the size of the list)
     */
    void set(int index, E elem);

    /**
     * Needed for the java.lang.Iterable interface. Implementation of the Iterator interface
     * and the iterator() method are left to the user.
     * 
     * @see LinkedList#iterator()
     * @see ArrayList#iterator()
     * 
     * @return Generic Iterator (interface) from java.util.Iterator.
     */
    Iterator<E> iterator();
}
